function [v_WT,  v_sol_minAV_flux, activity , status] = FBA_min_fluxAV(FBAmodel, verbose)
% Output
%        v_WT - The regular FBA maximum growth solution, that can have a
%               very large flux activity
%
%        v_sol_minAV_flux - minimize the total flux activity
%

    Mets = FBAmodel.mets;
    Rxns = FBAmodel.rxns;

    if ( isfield( FBAmodel, 'genes') )
        Genes = FBAmodel.genes;
    end
    if ( isfield( FBAmodel, 'genes') )
        RxnGeneMat = FBAmodel.rxnGeneMat;
    end

    S = FBAmodel.S;
    Obj = FBAmodel.c;
    Lbound = FBAmodel.lb;
    Ubound = FBAmodel.ub;

	%NUMBER OF METABOLITES and Reactions
	[M,N] = size(S);

	%SET THE B VECTOR FOR THE RIGHT HAND SIDE OF EQUATION, Sv= 0
	Bvector =   zeros(M,1); % right hand side vector for stoichiometric constr. S*v=0

	%GENERATES A STRING OF SYMBOLS 'S', INDICATING THE TYPE OF CONSTRAINTS (<,>,=)
	%(IN STIOCH*V = 0) USED IN THE OPTIMIZER. THE FOLLOWIN SETTING IS DEFAULT IN FBA
    CTYPE = char(83*ones(1,M))';

	%GENERATES A STRING OF SYMBOLS 'C', INDICATING THE TYPE OF VARIABLES
	%'C' MEANS CONTINUOUS VARIABLES (STANDARD IN FBA)
	VARTYPE = char(67*ones(1,N))';

	%IF SET TO 3, PRINTS OUT DETAILS OF OPTIMIZATION PROCESS
	param.msglev = 1;

	%default = 1: Revised simplex method
	param.LPsolver = 1; % for glpk:

	%IF 1, SAVE PROBLEM IN LP FORMAT
	param.save_pb = 0;

    %Max searching time in seconds
    param.tmlim = 600;

    %max number of iterations to solve
%     param.itlim = 10000;

    %set scaling to geometric
    param.scale = 2;  % default is 1  original(2)

    %***** Turn on presolver can keep away from rounding error *****
    param.presol = 1;

    % set LP to maximize/minimize if maximize parameter passed, else just maximize
	% -1 MAXIMIZES, 1 MINIMIZES
    min_max = -1;

    % Fix glpkmex bug, Lboud = Uboud gives unreachable solution
    % It seems that this extra trick cause problem in FBA ... but need to
    % double check systematically. Right now I use it only when necessary
%     if (strcmp(computer,'PCWIN')==1 )
%         indices_eq = (find((Ubound-Lbound)==0));
%         Ubound(indices_eq) = Ubound(indices_eq) + 1e-7;
%     end

	[v_WT,obj_sol,status,extra] = glpk(Obj , S , Bvector , Lbound , Ubound , CTYPE, VARTYPE , min_max , param);


    violateLBid = intersect(find((v_WT - Lbound) < -1e-6), find(FBAmodel.isEX) );

    %==== Start FBA Error handling ====
    % status = Status of the optimization.
    %               1 solution is undefined
    %               2 solution is feasible
    %               3 solution is infeasible
    %               4 no feasible solution exists
    %               5 solution is optimal
    %               6 solution is unbounded
    % More details can is listed at the bottom of this script
   if(status~=5  || ~isempty(violateLBid) )
    %if FBA fails retry with error bar 1e-7 ...
        param.presol = 0;
        param.itlim = -1;
        param.tmlim = -1;
        indices_eq = (find((Ubound-Lbound)==0));
        Ub = Ubound;
        Ub(indices_eq) = Ub(indices_eq) + 1e-7;
        [v_WT,obj_sol,status,extra]=glpk(Obj , S , Bvector , Lbound , Ub , CTYPE , VARTYPE , min_max , param);

        if (status ==5)
            disp(['glpk succeed with 1e-7 error tolerence, Vgro = '   num2str(obj_sol)   ]);
        else

           if(status ==4 )
               %Consider this an optimal solution with zero growth
                obj_sol = 0;  status = status;
                v_WT = 0*v_WT;
                disp(['            ==== glpk Status 4 --> No Growth ===='])
           else
                %still fail, see if ATPM is causing problem
                id_s = strmatch( 'ATPM' , Rxns ,'exact' ) ;
                Lbound0 = Lbound;         Lbound0(id_s) = 0;
                %for SEED
                id_s = strmatch( 'rxn11300' , Rxns ,'exact' ) ;
                Lbound0(id_s) = 0;
                Ubound0 = Ub;

                [v_WT0,obj_sol0,status0,extra0]=glpk(Obj , S , Bvector , Lbound0 , Ubound0 , CTYPE , VARTYPE , min_max , param);

                if(status0 ==5 || status0 ==4)
                   %W/O ATPM we get optimal solution,
                   %Consider this an optimal solution with zero growth
                   %
                   %If we wanna consider massive death we need to get this
                   %ATPM status, e.g. return v_ATPM or its status
                    obj_sol = 0;
                    status = 5;
                    v_WT = 0*v_WT;

                    disp(['            ==== Cannot support ATPM so set every flux to zero ===='])
                else
                    fold = 1e3;
                    [v_WT1,obj_sol1,status1,extra1]=rescaleglpk(fold, Obj , S , Bvector , Lbound , Ubound , CTYPE , VARTYPE , min_max , param);
                    sum(abs(v_WT1))

                    if(status1 ==5 )
                        v_WT = v_WT1; obj_sol = obj_sol1; status = status1; extra = extra1;
                        disp(['            ==== Scaling up 1e3 yields optimal solution ===='])
                    elseif(status1 ==4 )
                        obj_sol = 0;
                        status = 5;
                        v_WT = 0*v_WT;
                    else

                        obj_sol = 0;
                        status = 5;
                        v_WT = 0*v_WT;
% %
% %
% %                        status
% %                        status0
% %                        ex = find(FBAmodel.isEX);
% %                        [char(FBAmodel.rxnNames(ex)) num2str(FBAmodel.lb(ex))    num2str(v_WT(ex)) ]
% %
% %                        save('test.mat')
% %                        %disp([ Lbound0  Ubound])
% %                        %W/O ATPM we still get abnormal solution from FBA(glpk)
% %                        %*** PLACE TO IMPROVE THIS ***
% %                        %This is likely a no growth solution but you need to
% %                        %know the growth of the previous time point
% %                        error('         ==== glpk abnormal solution for FBA: Should not happen ====')
                    end
                end
            end
        end
   end
% % debug
    if (verbose)
        disp(['               glpk Growth = ' num2str(obj_sol )  '    activity = ' num2str(sum(abs(v_WT))) '('  FBAmodel.description  ')']) ;
    end
   %==== End FBA Error handling ====



   %[2] Now minimize total flux activity by fixing FBAgrowth,
   %    may need to relax this growht constraint with min_Flux_AV_relaxed.m
   if(obj_sol < 1e-3)
    %No growth => don't produce anything, everything is zero
    %This may be relaxed to 1e-6 since the major problem seems to be
    %glpk Lbound = Ubound bug in minimization of flux activity
    %status==4 => No feasible solution, unable to maintain ATPM, reset everything

        obj_sol = 0;
        v_WT = 0*v_WT;
        v_sol_minAV_flux = 0*v_WT;
        activity = 0;
        %return [v_WT,  v_sol_minAV_flux, activity , status]

   %FBAmodel grows and we want to look for minimum flux activity
   else

        FBAgrowth = obj_sol ;
        [v_sol_minAV_flux ,obj_sol_minAV,status_min_AV,extra] = min_Flux_AV(Obj, obj_sol, Rxns, Mets, S , Bvector , Lbound , Ubound , CTYPE, VARTYPE, verbose);

        if(status_min_AV ~=5)

            disp('minimization of flux activity failed : turn off presolver and try fixing potential glpkmex bug ')
            param.itlim = -1;
            param.tmlim = -1;
            param.presol = 0;

            %old glpkmex bug, but still useful in minimization of flux activity
            indices_eq = (find((Ubound-Lbound)==0));
            Ubound(indices_eq) = Ubound(indices_eq) + 1e-7;

            [v_WT,obj_sol,status,extra]=glpk(Obj , S , Bvector , Lbound , Ubound , CTYPE , VARTYPE , min_max , param);
            [v_sol_minAV_flux ,obj_sol_minAV,status_min_AV,extra] = min_Flux_AV(Obj, obj_sol, Rxns, Mets, S , Bvector , Lbound , Ubound , CTYPE, VARTYPE, verbose);

                if (status_min_AV ~=5)
                    disp('Try to relax FBA growth constraint on Mimimization of Flux Activity ...............')
                    [v_sol_minAV_flux ,obj_sol_minAV,status_min_AV,extra] = min_Flux_AV_relaxed(Obj, obj_sol, Rxns, Mets, S , Bvector , Lbound , Ubound , CTYPE, VARTYPE);

                    if (status_min_AV ~=5)
                        ex = find(FBAmodel.isEX);
                        [char(FBAmodel.rxns(ex)) num2str(FBAmodel.lb(ex))    num2str(v_WT(ex)) ]
                        save('msf.mat')
                        error('Nothing worked ... Study this case msf.mat')

                    end
                end
        end
        activity = sum(abs(v_sol_minAV_flux));
        status = mean(status, status_min_AV);
   end



%   return [v_WT,  v_sol_minAV_flux, activity , status]

end

function [v_WT1,obj_sol1,status1,extra1]=rescaleglpk(fold, Obj , S , Bvector , Lbound , Ubound , CTYPE , VARTYPE , min_max , param)
%This function increase the level and see if status 5 is reached
	Lbound1 = Lbound*fold;
	Ubound1 = Ubound*fold;
	[v_WT1,obj_sol1,status1,extra1]=glpk(Obj , S , Bvector , Lbound1 , Ubound1 , CTYPE , VARTYPE , min_max , param);

	v_WT1 = v_WT1/fold;
	obj_sol1 = obj_sol1/fold;

end