function [v_sol_MOMA_AV obj_sol status] = MOMA_AV(MOMAmodel, V_initial)
        S = MOMAmodel.S;
        Lbound = MOMAmodel.lb;
        Ubound = MOMAmodel.ub;
        Metabolites = MOMAmodel.mets;
        Reactions = MOMAmodel.rxns;
        OBJ = MOMAmodel.c;
        % Takes as input :
        %        S                 - The stoichiometric matrix, rows are metabolites and columns are reactions
        %        Ubound                - A vector the same length as the number of reactions, indicating the upper bound for each
        %        Lbound                - A vector the same length as the number of reactions, indicating the lower bound for each
        %        OBJ                - A vector the same length as the number of reactions, indicating the distance from which
        %                          fluxes should be minimized
        %        Metabolites        - A vector of metabolite names
        %        Reactions        - A vector of reaction names
        %        V_initial        - A vector the same length as OBJ with the values of the fluxes through each reaction
        %                          in the initial state
        %
        %
        % Returns : The fluxes through each reaction and the glpk status of the optimization. The fluxes are determined
        %            such that there is a minimum sum of absolute values of the flux vector from the V_initial vector


        %%%ADJUST STOICHIMOETRIC MATRIX AND B-VECTOR TO IMPOSE EXACT CONSTRAINTS%%%

        %SET THE B VECTOR FOR THE RIGHT HAND SIDE OF EQUATION, Sv= 0
        Bvector = zeros(size(S,1) , 1);


        %ADD ROWS TO S MATRIX AND ADJUST B-VECTOR IN ORDER TO IMPOSE SET FLUXES
        indices_eq = (find((Ubound-Lbound)==0));

        new_rows = zeros( length(indices_eq) , size(S,2));
        constr_rxns = sub2ind( [length(indices_eq) size(S,2)] , (1:length(indices_eq)) , indices_eq' );
        new_rows(constr_rxns) = 1;

        S = [S ; new_rows];
        Bvector = [Bvector ; Ubound(indices_eq)];


        %STILL HAVE TO ADD BUFFER TO BOUND EVEN THOUGHT EQUAL CONSTRAINTS ARE IMPOSED EXPLICITY, SO GLPMEX DOESN'T CHOKE
        Ubound(indices_eq) = Ubound(indices_eq) + 1e-7;



        %%%ADD EXTRA CONSTRAINTS SUCH THAT THE MINIMUM ABSOLUTE VALUE DISTANCE FROM V_INITIAL CAN BE CALCULATED

        %ADJUST S MATRIX TO ADD A NEW VARIABLE CORRESPODING TO EACH FLUX, WHICH WILL FUNCTION AS A DISTANCE FROM ZERO
        %OF THE DIFFERENCE BETWEEN A GIVEN FLUX AND ITS INTIAL VALUE
        new_variables = zeros( size(S,1) , size(S,2) );
        S = [S new_variables];


        %NUMBER OF METABOLITES PLUS ADDITIONAL CONSTRAINTS(ROWS OF S, CONSTRAINTS)
        M = size(S,1);

        %NUMBER OF REACTIONS AND ADDITION VARIABLES(COLUMNS OF S, VARIABLES)
        N = size(S,2);


        %ADD TWO NEW CONSTRAINTS FOR EACH FLUX TO ACCOUNT FOR THE CASES OF THE FLUX BEING LESS THAN OR GREATER THAN ZERO
        new_constraints = zeros( size(S,2) , size(S,2) );

        upper_const = 1:2:size(new_constraints,1);
        lower_const = 2:2:size(new_constraints,1);

        fluxes_to_const = 1:(N/2);
        const_vars = fluxes_to_const + (N/2);

        upper_const_flux_i = sub2ind([size(new_constraints,1) size(new_constraints,2)] , upper_const , fluxes_to_const);
        upper_const_var_i = sub2ind([size(new_constraints,1) size(new_constraints,2)] , upper_const , const_vars);
        new_constraints(upper_const_flux_i) = 1;
        new_constraints(upper_const_var_i) = -1;

        lower_const_flux_i = sub2ind([size(new_constraints,1) size(new_constraints,2)] , lower_const , fluxes_to_const);
        lower_const_var_i = sub2ind([size(new_constraints,1) size(new_constraints,2)] , lower_const , const_vars);
        new_constraints(lower_const_flux_i) = -1;
        new_constraints(lower_const_var_i) = -1;

        new_thresholds = zeros( size(V_initial , 1) , 1 );
        new_thresholds(upper_const) = V_initial;
        new_thresholds(lower_const) = -1 * V_initial;

% I think this would allow it to search outside the bound
% new_thresholds(upper_const) = Ubound;
% new_thresholds(lower_const) = Lbound;


        Bvector = [Bvector ; new_thresholds];
        S = [S ; new_constraints];


        %SET NEW OBJECTIVE WHICH MINIMIZES THE SUM OF THE NEW VARIABLES REPRESENTING EACH OF THE FLUXES DISTANCE FROM ZERO
        OBJ = [zeros( length(OBJ) , 1) ; ones( length(OBJ) , 1)];
        Lbound = [Lbound ; zeros( (N/2) , 1)];
        Ubound = [Ubound ; Inf( (N/2) , 1)];


        % -1 MAXIMIZES, 1 MINIMIZES
        %MINIMIZE SUM OF ABSOLUTE VALUES OF FLUXES
        min_max = 1;


        %GENERATES A STRING OF SYMBOLS 'S' and 'U', INDICATING THE TYPE OF CONSTRAINTS (<,>,=)
        %(IN STIOCH*V = 0) USED IN THE OPTIMIZER. THE FOLLOWIN SETTING IS DEFAULT IN FBA
        %***C-TYPE FOR NEW CONSTRAINTS HAS UPPER BOUND***
        CTYPE = [ char( 83*ones(1,M) )' ; char( 85*ones( 1,size(new_constraints,1)) )'];
        %               'S'                     'U'

        %GENERATES A STRING OF SYMBOLS 'C', INDICATING THE TYPE OF VARIABLES
        %'C' MEANS CONTINUOUS VARIABLES (STANDARD IN FBA)
        %VARTYPES STILL ALL CONTINUOUS
        VARTYPE = char( 67*ones(1,size(new_constraints,2)) )';


        %IF SET TO 3, PRINTS OUT DETAILS OF OPTIMIZATION PROCESS
        param.msglev = 1;

% % THESE ARE COPIED FROM FBA.m trying to solve the scaling warning message
    %set scaling to geometric
    param.scale = 2;  % default is 1  original(2)

        %Max searching time in seconds
        %    param.tmlim = 60;
        % %max number of iterations to solve
        param.itlim = 10000;

        %default = 1: Revised simplex method
        param.LPsolver = 1; % for glpk

% % THESE ARE COPIED FROM FBA.m trying to solve the scaling warning message


        %IF 1, SAVE PROBLEM IN LP FORMAT
        param.save_pb = 0;  % for glpk

        tmp = zeros(1, size(new_variables,2));
        if (isfield(MOMAmodel,'extra_row_of_S') ==1)
            tmp = [MOMAmodel.extra_row_of_S   tmp];
            S =[S ; tmp];
            % e.g. 0.1Vgro - EX_ade(e) <= 0
            CTYPE(end+1 : end+size(MOMAmodel.extra_row_of_S,1))='U';
            Bvector(end+1 : end+size(MOMAmodel.extra_row_of_S,1))=0;
        end

        %PERFORM OPTIMIZATION
        [v_sol , obj_sol , status ,extra] = glpk(OBJ,S,Bvector,Lbound,Ubound,CTYPE,VARTYPE,min_max,param);

        if (status ~= 5 )
            %TURN OFF THE PRESOLVER and try again
            param.presol = 0;
            param.itlim = 100000;
            [v_sol , obj_sol , status ,extra] = glpk(OBJ,S,Bvector,Lbound,Ubound,CTYPE,VARTYPE,min_max,param);
            disp('Relaxing iteration limit to 100000');
            if(status ~= 5 )
               error('Still fail to get MOMA_AV results') ;

            end
        end

        v_sol_MOMA_AV = v_sol( 1:size(Reactions,1) );
%     status
%     extra
%     disp(status)